@@ -1,26 +1,27 @@
-/*
-
-Copyright (C) 2013-2018 John W. Eaton
-Copyright (C) 2015 Michael Barnes
-Copyright (C) 2013 Rüdiger Sonderfeld
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation, either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2013-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -40,52 +41,62 @@ along with Octave; see the file COPYING.  If not, see
 #include <QScrollBar>
 #include <QSignalMapper>
 #include <QStackedWidget>
+#include <QTabWidget>
 #include <QTableView>
 #include <QTextEdit>
-#include <QTabWidget>
 #include <QToolBar>
 #include <QToolButton>
 #include <QVBoxLayout>
 
-#include "resource-manager.h"
+#include "dw-main-window.h"
+#include "gui-preferences-cs.h"
+#include "gui-preferences-global.h"
+#include "gui-preferences-ve.h"
+#include "octave-qobject.h"
 #include "shortcut-manager.h"
-#include "variable-editor.h"
 #include "variable-editor-model.h"
-
-// Code reuse functions
-
-static QString
-idx_to_expr (int32_t from, int32_t to)
-{
-  return (from == to
-          ? QString ("%1").arg (from)
-          : QString ("%1:%2").arg (from).arg (to));
-}
-
-static QSignalMapper *
-make_plot_mapper (QMenu *menu)
-{
-  QList<QString> list;
-  list << "plot" << "bar" << "stem" << "stairs" << "area" << "pie" << "hist";
-
-  QSignalMapper *plot_mapper = new QSignalMapper (menu);
-
-  for (int i = 0; i < list.size(); ++i)
-    {
-      plot_mapper->setMapping
-        (menu->addAction (list.at (i), plot_mapper, SLOT (map ())),
-         "figure (); " + list.at (i) + " (%1); title (\"%1\");");
-    }
-
-  return plot_mapper;
-}
+#include "variable-editor.h"
 
 namespace octave
 {
+  // Code reuse functions
+
+  static QString
+  idx_to_expr (int32_t from, int32_t to)
+  {
+    return (from == to
+            ? QString ("%1").arg (from)
+            : QString ("%1:%2").arg (from).arg (to));
+  }
+
+  static QSignalMapper *
+  make_plot_mapper (QMenu *menu)
+  {
+    QList<QString> list;
+    list << "plot" << "bar" << "stem" << "stairs" << "area" << "pie" << "hist";
+
+    QSignalMapper *plot_mapper = new QSignalMapper (menu);
+
+    for (int i = 0; i < list.size(); ++i)
+      {
+        plot_mapper->setMapping
+          (menu->addAction (list.at (i), plot_mapper, SLOT (map ())),
+           "figure (); " + list.at (i) + " (%1); title (\"%1\");");
+      }
+
+    return plot_mapper;
+  }
+
   // Variable dock widget
 
-  variable_dock_widget::variable_dock_widget (QWidget *p)
-    : label_dock_widget (p)
+  variable_dock_widget::variable_dock_widget (QWidget *p,
+                                              base_qobject& oct_qobj)
+    : label_dock_widget (p, oct_qobj)
+// See  Octave bug #53807 and https://bugreports.qt.io/browse/QTBUG-44813
+#if (QT_VERSION >= 0x050302) && (QT_VERSION <= QTBUG_44813_FIX_VERSION)
+      , m_waiting_for_mouse_move (false)
+      , m_waiting_for_mouse_button_release (false)
+#endif
   {
     setFocusPolicy (Qt::StrongFocus);
     setAttribute (Qt::WA_DeleteOnClose);
@@ -109,28 +120,26 @@ namespace octave
     m_prev_floating = false;
     m_prev_geom = QRect (0, 0, 0, 0);
 
-    QHBoxLayout *h_layout = findChild<QHBoxLayout *> ();
-    if (h_layout != nullptr && titleBarWidget () != nullptr)
-      {
-        m_fullscreen_action = new QAction
-          (resource_manager::icon ("view-fullscreen", false), "", this);
-        m_fullscreen_action->setToolTip (tr (DOCKED_FULLSCREEN_BUTTON_TOOLTIP));
-        QToolButton *fullscreen_button = new QToolButton (titleBarWidget ());
-        fullscreen_button->setDefaultAction (m_fullscreen_action);
-        fullscreen_button->setFocusPolicy (Qt::NoFocus);
-        fullscreen_button->setIconSize (QSize (m_icon_size,m_icon_size));
-        QString css_button = QString ("QToolButton {background: transparent; border: 0px;}");
-        fullscreen_button->setStyleSheet (css_button);
+    QHBoxLayout *h_layout = m_title_widget->findChild<QHBoxLayout *> ();
+    resource_manager& rmgr = m_octave_qobj.get_resource_manager ();
+    m_fullscreen_action
+      = new QAction (rmgr.icon ("view-fullscreen", false), "", this);
+    m_fullscreen_action->setToolTip (tr (DOCKED_FULLSCREEN_BUTTON_TOOLTIP));
+    QToolButton *fullscreen_button = new QToolButton (m_title_widget);
+    fullscreen_button->setDefaultAction (m_fullscreen_action);
+    fullscreen_button->setFocusPolicy (Qt::NoFocus);
+    fullscreen_button->setIconSize (QSize (m_icon_size,m_icon_size));
+    QString css_button = QString ("QToolButton {background: transparent; border: 0px;}");
+    fullscreen_button->setStyleSheet (css_button);
 
-        connect (m_fullscreen_action, SIGNAL (triggered ()),
-                 this, SLOT (change_fullscreen ()));
+    connect (m_fullscreen_action, SIGNAL (triggered ()),
+             this, SLOT (change_fullscreen ()));
 
-        int index = -1;
-        QToolButton *first = titleBarWidget ()->findChild<QToolButton *> ();
-        if (first != nullptr)
-          index = h_layout->indexOf (first);
-        h_layout->insertWidget (index, fullscreen_button);
-      }
+    int index = -1;
+    QToolButton *first = m_title_widget->findChild<QToolButton *> ();
+    if (first != nullptr)
+      index = h_layout->indexOf (first);
+    h_layout->insertWidget (index, fullscreen_button);
 #endif
 
     // Custom title bars cause loss of decorations, add a frame
@@ -149,7 +158,8 @@ namespace octave
         if (m_full_screen)
           {
             setGeometry (m_prev_geom);
-            m_fullscreen_action->setIcon (resource_manager::icon ("view-fullscreen", false));
+            resource_manager& rmgr = m_octave_qobj.get_resource_manager ();
+            m_fullscreen_action->setIcon (rmgr.icon ("view-fullscreen", false));
             m_full_screen = false;
           }
         m_fullscreen_action->setToolTip (tr (DOCKED_FULLSCREEN_BUTTON_TOOLTIP));
@@ -176,15 +186,30 @@ namespace octave
         m_dock_action->setIcon (QIcon (":/actions/icons/widget-dock.png"));
         m_dock_action->setToolTip (tr ("Dock widget"));
 
-        activateWindow();
-        setFocus (Qt::OtherFocusReason);
+        setWindowFlags (Qt::Window);
+        setWindowTitle (tr ("Variable Editor: ") + objectName ());
+
+        show ();
+        activateWindow ();
+        setFocus ();
+
+// See  Octave bug #53807 and https://bugreports.qt.io/browse/QTBUG-44813
+#if (QT_VERSION >= 0x050302) && (QT_VERSION <= QTBUG_44813_FIX_VERSION)
+        m_waiting_for_mouse_move = true;
+#endif
       }
     else
       {
         m_dock_action->setIcon (QIcon (":/actions/icons/widget-undock.png"));
         m_dock_action->setToolTip (tr ("Undock widget"));
 
-        setFocus (Qt::OtherFocusReason);
+        setFocus ();
+
+// See  Octave bug #53807 and https://bugreports.qt.io/browse/QTBUG-44813
+#if (QT_VERSION >= 0x050302) && (QT_VERSION <= QTBUG_44813_FIX_VERSION)
+        m_waiting_for_mouse_move = false;
+        m_waiting_for_mouse_button_release = false;
+#endif
       }
   }
 
@@ -192,12 +217,12 @@ namespace octave
   variable_dock_widget::change_fullscreen (void)
   {
 #if defined (HAVE_QGUIAPPLICATION)
+    resource_manager& rmgr = m_octave_qobj.get_resource_manager ();
+
     if (! m_full_screen)
       {
         m_prev_floating = isFloating ();
-        m_prev_geom = geometry ();
-
-        m_fullscreen_action->setIcon (resource_manager::icon ("view-restore", false));
+        m_fullscreen_action->setIcon (rmgr.icon ("view-restore", false));
         if (m_prev_floating)
           m_fullscreen_action->setToolTip (tr ("Restore geometry"));
         else
@@ -205,6 +230,7 @@ namespace octave
             m_fullscreen_action->setToolTip (tr ("Redock"));
             setFloating (true);
           }
+        m_prev_geom = geometry ();
 
         // showFullscreen() and setWindowState() only work for QWindow objects.
         QScreen *pscreen = QGuiApplication::primaryScreen ();
@@ -216,7 +242,7 @@ namespace octave
       }
     else
       {
-        m_fullscreen_action->setIcon (resource_manager::icon ("view-fullscreen", false));
+        m_fullscreen_action->setIcon (rmgr.icon ("view-fullscreen", false));
         setGeometry (m_prev_geom);
         if (m_prev_floating)
           m_fullscreen_action->setToolTip (tr (UNDOCKED_FULLSCREEN_BUTTON_TOOLTIP));
@@ -244,23 +270,35 @@ namespace octave
   {
     octave_unused_parameter (now);
 
-    // The is a proxied test
+    // This is a proxied test
     if (hasFocus ())
       {
-        QLabel *label = titleBarWidget ()->findChild<QLabel *> ();
-        if (label != nullptr)
+        if (old == this)
+          return;
+
+        if (titleBarWidget () != nullptr)
           {
-            label->setBackgroundRole (QPalette::Highlight);
-            label->setAutoFillBackground (true);
+            QLabel *label = titleBarWidget ()->findChild<QLabel *> ();
+            if (label != nullptr)
+              {
+                label->setBackgroundRole (QPalette::Highlight);
+                label->setStyleSheet ("background-color: palette(highlight); color: palette(highlightedText);");
+              }
           }
 
         emit variable_focused_signal (objectName ());
       }
     else if (old == focusWidget())
       {
-        QLabel *label = titleBarWidget ()->findChild<QLabel *> ();
-        if (label != NULL)
-          label->setBackgroundRole (QPalette::NoRole);
+        if (titleBarWidget () != nullptr)
+          {
+            QLabel *label = titleBarWidget ()->findChild<QLabel *> ();
+            if (label != nullptr)
+              {
+                label->setBackgroundRole (QPalette::NoRole);
+                label->setStyleSheet (";");
+              }
+          }
       }
   }
 
@@ -270,11 +308,77 @@ namespace octave
       m_frame->resize (size ());
   }
 
+// See  Octave bug #53807 and https://bugreports.qt.io/browse/QTBUG-44813
+#if (QT_VERSION >= 0x050302) && (QT_VERSION <= QTBUG_44813_FIX_VERSION)
+
+  bool
+  variable_dock_widget::event (QEvent *event)
+  {
+    // low-level check of whether docked-widget became a window via
+    // via drag-and-drop
+    if (event->type () == QEvent::MouseButtonPress)
+      {
+        m_waiting_for_mouse_move = false;
+        m_waiting_for_mouse_button_release = false;
+      }
+    if (event->type () == QEvent::MouseMove && m_waiting_for_mouse_move)
+      {
+        m_waiting_for_mouse_move = false;
+        m_waiting_for_mouse_button_release = true;
+      }
+    if (event->type () == QEvent::MouseButtonRelease
+        && m_waiting_for_mouse_button_release)
+      {
+        m_waiting_for_mouse_button_release = false;
+        bool retval = QDockWidget::event (event);
+        if (isFloating ())
+          emit queue_unfloat_float ();
+        return retval;
+      }
+
+    return QDockWidget::event (event);
+  }
+
+  void
+  variable_dock_widget::unfloat_float (void)
+  {
+    hide ();
+    setFloating (false);
+    // Avoid a Ubunty Unity issue by queuing this rather than direct.
+    emit queue_float ();
+    m_waiting_for_mouse_move = false;
+    m_waiting_for_mouse_button_release = false;
+  }
+
+  void
+  variable_dock_widget::refloat (void)
+  {
+    setFloating (true);
+    m_waiting_for_mouse_move = false;
+    m_waiting_for_mouse_button_release = false;
+    show ();
+    activateWindow ();
+    setFocus ();
+  }
+
+#else
+
+  void
+  variable_dock_widget::unfloat_float (void)
+  { }
+
+  void
+  variable_dock_widget::refloat (void)
+  { }
+
+#endif
 
   // Variable editor stack
 
-  variable_editor_stack::variable_editor_stack (QWidget *p)
-    : QStackedWidget (p), m_edit_view (new variable_editor_view (this))
+  variable_editor_stack::variable_editor_stack (QWidget *p,
+                                                base_qobject& oct_qobj)
+    : QStackedWidget (p), m_octave_qobj (oct_qobj),
+      m_edit_view (new variable_editor_view (this, m_octave_qobj))
   {
     setFocusPolicy (Qt::StrongFocus);
 
@@ -344,7 +448,7 @@ namespace octave
 
     if (name.endsWith (')') || name.endsWith ('}'))
       {
-        name.remove (QRegExp ("(\\(|\\{)[^({]*(\\)|\\})$"));
+        name.remove ( QRegExp ("[({][^({]*[)}]$)") );
         emit edit_variable_signal (name, octave_value ());
       }
   }
@@ -355,14 +459,20 @@ namespace octave
     if (! hasFocus ())
       return;
 
+    // FIXME: Remove, if for all common KDE versions (bug #54607) is resolved.
+    int opts = 0;  // No options by default.
+    resource_manager& rmgr = m_octave_qobj.get_resource_manager ();
+    gui_settings *settings = rmgr.get_settings ();
+    if (! settings->value (global_use_native_dialogs).toBool ())
+      opts = QFileDialog::DontUseNativeDialog;
+
     QString name = objectName ();
     QString file
       = QFileDialog::getSaveFileName (this,
                                       tr ("Save Variable %1 As").arg (name),
     // FIXME: Should determine extension from save_default_options
                                       QString ("./%1.txt").arg (name),
-                                      0, 0,
-                                      QFileDialog::DontUseNativeDialog);
+                                      0, 0, QFileDialog::Option (opts));
 
     // FIXME: Type? binary, float-binary, ascii, text, hdf5, matlab format?
     // FIXME: Call octave_value::save_* directly?
@@ -376,8 +486,9 @@ namespace octave
 
   // Custom editable variable table view
 
-  variable_editor_view::variable_editor_view (QWidget *p)
-    : QTableView (p), m_var_model (nullptr)
+  variable_editor_view::variable_editor_view (QWidget *p,
+                                              base_qobject& oct_qobj)
+    : QTableView (p), m_octave_qobj (oct_qobj), m_var_model (nullptr)
   {
     setWordWrap (false);
     setContextMenuPolicy (Qt::CustomContextMenu);
@@ -483,35 +594,31 @@ namespace octave
   variable_editor_view::add_edit_actions (QMenu *menu,
                                           const QString& qualifier_string)
   {
-    menu->addAction (resource_manager::icon ("edit-cut"),
+    resource_manager& rmgr = m_octave_qobj.get_resource_manager ();
+
+    menu->addAction (rmgr.icon ("edit-cut"),
                      tr ("Cut") + qualifier_string,
                      this, SLOT (cutClipboard ()));
 
-    menu->addAction (resource_manager::icon ("edit-copy"),
+    menu->addAction (rmgr.icon ("edit-copy"),
                      tr ("Copy") + qualifier_string,
                      this, SLOT (copyClipboard ()));
 
-    menu->addAction (resource_manager::icon ("edit-paste"),
+    menu->addAction (rmgr.icon ("edit-paste"),
                      tr ("Paste"),
                      this, SLOT (pasteClipboard ()));
 
-    // FIXME: Different icon for Paste Table?
-
-    menu->addAction (resource_manager::icon ("edit-paste"),
-                     tr ("Paste Table"),
-                     this, SLOT (pasteTableClipboard ()));
-
     menu->addSeparator ();
 
-    menu->addAction (resource_manager::icon ("edit-delete"),
+    menu->addAction (rmgr.icon ("edit-delete"),
                      tr ("Clear") + qualifier_string,
                      this, SLOT (clearContent ()));
 
-    menu->addAction (resource_manager::icon ("edit-delete"),
+    menu->addAction (rmgr.icon ("edit-delete"),
                      tr ("Delete") + qualifier_string,
                      this, SLOT (delete_selected ()));
 
-    menu->addAction (resource_manager::icon ("document-new"),
+    menu->addAction (rmgr.icon ("document-new"),
                      tr ("Variable from Selection"),
                      this, SLOT (createVariable ()));
   }
@@ -577,12 +684,10 @@ namespace octave
       {
         selectColumn (index);
         column_selection_count = 1;
-        current_column_selected = true;
-        whole_columns_selected = true;
       }
 
     QString column_string
-      = tr (column_selection_count > 1 ? " columns" : " column");
+      = column_selection_count > 1 ? tr (" columns") : tr (" column");
 
     QMenu *menu = new QMenu (this);
 
@@ -627,11 +732,9 @@ namespace octave
       {
         selectRow (index);
         rowselection_count = 1;
-        current_row_selected = true;
-        whole_rows_selected = true;
       }
 
-    QString row_string = tr (rowselection_count > 1 ? " rows" : " row");
+    QString row_string = rowselection_count > 1 ? tr (" rows") : tr (" row");
 
     QMenu *menu = new QMenu (this);
 
@@ -747,7 +850,7 @@ namespace octave
     QModelIndex previous = indices.first ();
     QString copy = mod->data (previous).toString ();
     indices.removeFirst ();
-    foreach (QModelIndex idx, indices)
+    for (auto idx : indices)
       {
         copy.push_back (previous.row () != idx.row () ? '\n' : '\t');
         copy.append (mod->data (idx).toString ());
@@ -771,38 +874,6 @@ namespace octave
     QClipboard *clipboard = QApplication::clipboard ();
     QString text = clipboard->text ();
 
-    if (indices.isEmpty ())
-      {
-        if (size () == QSize (1,1))
-          mod->setData (mod->index (0,0), text.toDouble ());
-        else if (size () == QSize (0,0))
-          {
-            mod->insertColumn (0);
-            mod->insertRow (0);
-            mod->setData (mod->index (0,0), text.toDouble ());
-          }
-      }
-    else
-      {
-        QStringList cells = text.split(QRegExp("\n|\r\n|\r"));
-        int clen = cells.size ();
-        for (int i = 0; i < indices.size (); i++)
-          mod->setData (indices[i], cells.at (i % clen).toDouble ());
-      }
-  }
-
-  void variable_editor_view::pasteTableClipboard (void)
-  {
-    if (! hasFocus ())
-      return;
-
-    QAbstractItemModel *mod = model ();
-    QItemSelectionModel *sel = selectionModel ();
-    QList<QModelIndex> indices = sel->selectedIndexes ();
-
-    QClipboard *clipboard = QApplication::clipboard ();
-    QString text = clipboard->text ();
-
     QPoint start, end;
 
     QPoint tabsize = QPoint (mod->rowCount (), mod->columnCount ());
@@ -927,8 +998,7 @@ namespace octave
   {
     if (ev->type () == QEvent::HoverEnter)
       emit hovered_signal ();
-    else if (ev->type () == QEvent::MouseButtonPress ||
-             ev->type () == QEvent::MouseButtonPress)
+    else if (ev->type () == QEvent::MouseButtonPress)
       emit popup_shown_signal ();
 
     return QToolButton::eventFilter (obj, ev);
@@ -976,9 +1046,9 @@ namespace octave
 
   // Variable editor.
 
-  variable_editor::variable_editor (QWidget *p)
-    : octave_dock_widget (p),
-      m_main (new QMainWindow ()),
+  variable_editor::variable_editor (QWidget *p, base_qobject& oct_qobj)
+    : octave_dock_widget ("VariableEditor", p, oct_qobj),
+      m_main (new dw_main_window (oct_qobj)),
       m_tool_bar (new QToolBar (m_main)),
       m_default_width (30),
       m_default_height (100),
@@ -990,9 +1060,10 @@ namespace octave
       m_sel_font (),
       m_table_colors (),
       m_current_focus_vname (""),
-      m_hovered_focus_vname ("")
+      m_hovered_focus_vname (""),
+      m_focus_widget (nullptr),
+      m_focus_widget_vdw (nullptr)
   {
-    setObjectName ("VariableEditor");
     set_title (tr ("Variable Editor"));
     setStatusTip (tr ("Edit variables."));
     setWindowIcon (QIcon (":/actions/icons/logo.png"));
@@ -1015,7 +1086,7 @@ namespace octave
 
     // Colors.
 
-    for (int i = 0; i < resource_manager::varedit_color_chars ().length (); i++)
+    for (int i = 0; i < ve_colors_count; i++)
       m_table_colors.append (QColor (Qt::white));
 
     // Use an MDI area that is shrunk to nothing as the central widget.
@@ -1038,10 +1109,34 @@ namespace octave
   {
     octave_dock_widget::focusInEvent (ev);
 
-    // set focus to the current variable
-    QWidget *fw = m_main->focusWidget ();
-    if (fw != nullptr)
-      fw->setFocus ();
+    // set focus to the current variable or most recent if still valid
+    if (m_focus_widget != nullptr)
+      {
+        // Activating a floating window causes problems.
+        if (! m_focus_widget_vdw->isFloating ())
+          activateWindow ();
+        m_focus_widget->setFocus ();
+      }
+    else
+      {
+        QWidget *fw = m_main->focusWidget ();
+        if (fw != nullptr)
+          {
+            activateWindow ();
+            fw->setFocus ();
+          }
+        else
+          {
+            QDockWidget *any_qdw = m_main->findChild<QDockWidget *> ();
+            if (any_qdw != nullptr)
+              {
+                activateWindow ();
+                any_qdw->setFocus ();
+              }
+            else
+              setFocus();
+          }
+      }
   }
 
   // Add an action to a menu or the widget itself.
@@ -1070,9 +1165,11 @@ namespace octave
   void
   variable_editor::edit_variable (const QString& name, const octave_value& val)
   {
+    resource_manager& rmgr = m_octave_qobj.get_resource_manager ();
+
     if (m_stylesheet.isEmpty ())
       {
-        QSettings *settings = resource_manager::get_settings ();
+        gui_settings *settings = rmgr.get_settings ();
         notice_settings (settings);
       }
 
@@ -1099,7 +1196,9 @@ namespace octave
         return;
       }
 
-    variable_dock_widget *page = new variable_dock_widget (this);
+    variable_dock_widget *page
+      = new variable_dock_widget (this, m_octave_qobj);
+
     page->setObjectName (name);
     m_main->addDockWidget (Qt::LeftDockWidgetArea, page);
 
@@ -1109,8 +1208,17 @@ namespace octave
              this, SLOT (variable_destroyed (QObject *)));
     connect (page, SIGNAL (variable_focused_signal (const QString&)),
              this, SLOT (variable_focused (const QString&)));
+// See  Octave bug #53807 and https://bugreports.qt.io/browse/QTBUG-44813
+#if (QT_VERSION >= 0x050302) && (QT_VERSION <= QTBUG_44813_FIX_VERSION)
+    connect (page, SIGNAL (queue_unfloat_float ()),
+             page, SLOT (unfloat_float ()), Qt::QueuedConnection);
+    connect (page, SIGNAL (queue_float ()),
+             page, SLOT (refloat ()), Qt::QueuedConnection);
+#endif
+
+    variable_editor_stack *stack
+      = new variable_editor_stack (page, m_octave_qobj);
 
-    variable_editor_stack *stack = new variable_editor_stack (page);
     stack->setObjectName (name);
     page->setWidget (stack);
     page->setFocusProxy (stack);
@@ -1143,8 +1251,6 @@ namespace octave
              edit_view, SLOT (copyClipboard ()));
     connect (this, SIGNAL (paste_clipboard_signal ()),
              edit_view, SLOT (pasteClipboard ()));
-    connect (this, SIGNAL (paste_table_clipboard_signal ()),
-             edit_view, SLOT (pasteTableClipboard ()));
     connect (this, SIGNAL (selected_command_signal (const QString&)),
              edit_view, SLOT (selected_command_requested (const QString&)));
     connect (edit_view->horizontalHeader (),
@@ -1176,15 +1282,27 @@ namespace octave
     connect (edit_view, SIGNAL (doubleClicked (const QModelIndex&)),
              model, SLOT (double_click (const QModelIndex&)));
 
+    // Any interpreter_event signal from a variable_editor_model object is
+    // handled the same as for the parent variable_editor object.
+
+    connect (model, SIGNAL (interpreter_event (const fcn_callback&)),
+             this, SIGNAL (interpreter_event (const fcn_callback&)));
+
+    connect (model, SIGNAL (interpreter_event (const meth_callback&)),
+             this, SIGNAL (interpreter_event (const meth_callback&)));
+
     // Must supply a title for a QLabel to be created.  Calling set_title()
     // more than once will add more QLabels.  Could change octave_dock_widget
-    // to always supply a QLabl (initially empty) and then simply update its
+    // to always supply a QLabel (initially empty) and then simply update its
     // contents.
     page->set_title (name);
-    QLabel *existing_ql = page->titleBarWidget ()->findChild<QLabel *> ();
-    connect (model, SIGNAL (update_label_signal (const QString&)),
-             existing_ql, SLOT (setText (const QString&)));
-    existing_ql->setMargin (2);
+    if (page->titleBarWidget () != nullptr)
+      {
+        QLabel *existing_ql = page->titleBarWidget ()->findChild<QLabel *> ();
+        connect (model, SIGNAL (update_label_signal (const QString&)),
+                 existing_ql, SLOT (setText (const QString&)));
+        existing_ql->setMargin (2);
+      }
 
     model->update_data (val);
 
@@ -1208,7 +1326,7 @@ namespace octave
         QList<QTabBar *> barlist = main_win ()->findChildren<QTabBar *> ();
         QVariant this_value (reinterpret_cast<quintptr> (this));
 
-        foreach (QTabBar *tbar, barlist)
+        for (auto *tbar : barlist)
           for (int i = 0; i < tbar->count (); i++)
             if (tbar->tabData (i) == this_value)
               {
@@ -1224,34 +1342,6 @@ namespace octave
     emit refresh_signal ();
   }
 
-  QList<QColor>
-  variable_editor::default_colors (void)
-  {
-    QList<QColor> colorlist;
-
-    colorlist << qApp->palette ().color (QPalette::WindowText);
-    colorlist << qApp->palette ().color (QPalette::Base);
-    colorlist << qApp->palette ().color (QPalette::HighlightedText);
-    colorlist << qApp->palette ().color (QPalette::Highlight);
-    colorlist << qApp->palette ().color (QPalette::AlternateBase);
-
-    return colorlist;
-  }
-
-  QStringList
-  variable_editor::color_names (void)
-  {
-    QStringList output;
-
-    output << "Foreground";
-    output << "Background";
-    output << "Selected Foreground";
-    output << "Selected Background";
-    output << "Alternate Background";
-
-    return output;
-  }
-
   void
   variable_editor::callUpdate (const QModelIndex&, const QModelIndex&)
   {
@@ -1259,41 +1349,31 @@ namespace octave
   }
 
   void
-  variable_editor::notice_settings (const QSettings *settings)
+  variable_editor::notice_settings (const gui_settings *settings)
   {
-    // FIXME: Why use object->tostring->toint?  Why not just 100?
+    m_main->notice_settings (settings); // update settings in parent main win
 
-    m_default_width = settings->value ("variable_editor/column_width",
-                                       100).toInt ();
+    m_default_width = settings->value (ve_column_width).toInt ();
 
-    m_default_height = settings->value ("variable_editor/row_height",
-                                        10).toInt ();
+    m_default_height = settings->value (ve_row_height).toInt ();
 
-    m_alternate_rows = settings->value ("variable_editor/alternate_rows",
-                                        false).toBool ();
+    m_alternate_rows = settings->value (ve_alternate_rows).toBool ();
 
-    QList<QColor> default_colors = resource_manager::varedit_default_colors ();
-
-    QString class_chars = resource_manager::varedit_color_chars ();
-
-    m_use_terminal_font = settings->value ("variable_editor/use_terminal_font",
-                                           true).toBool ();
+    m_use_terminal_font = settings->value (ve_use_terminal_font).toBool ();
 
     QString font_name;
     int font_size;
+    QString default_font = settings->value (global_mono_font).toString ();
 
     if (m_use_terminal_font)
       {
-        font_name = settings->value ("terminal/fontName", "Courier New").toString ();
-        font_size = settings->value ("terminal/fontSize", 10).toInt ();
+        font_name = settings->value (cs_font.key, default_font).toString ();
+        font_size = settings->value (cs_font_size).toInt ();
       }
     else
       {
-        font_name = settings->value ("variable_editor/font_name",
-                                     settings->value ("terminal/fontName",
-                                                      "Courier New")).toString ();
-
-        font_size = settings->value ("variable_editor/font_size", 10).toInt ();
+        font_name = settings->value (ve_font_name.key, default_font).toString ();
+        font_size = settings->value (ve_font_size).toInt ();
       }
 
     m_font = QFont (font_name, font_size);
@@ -1302,37 +1382,30 @@ namespace octave
 
     m_add_font_height = fm.height ();
 
-    for (int i = 0; i < class_chars.length (); i++)
+    for (int i = 0; i < ve_colors_count; i++)
       {
-        QVariant default_var;
-        if (i < default_colors.length ())
-          default_var = default_colors.at (i);
-        else
-          default_var = QColor ();
+        // The default colors are given as color roles for
+        // the application's palette
+        QColor default_color = qApp->palette ().color
+                               (static_cast<QPalette::ColorRole> (ve_colors[i].def.toInt ()));
+        // FIXME: use value<QPalette::ColorRole> instead of static cast after
+        //        dropping support of Qt 5.4
 
-        QColor setting_color = settings->value ("variable_editor/color_"
-                                                + class_chars.mid (i, 1),
-                                                default_var).value<QColor> ();
+        QColor setting_color =
+          settings->value (ve_colors[i].key, default_color).value<QColor> ();
 
-        if (i < m_table_colors.length ())
-          m_table_colors.replace (i, setting_color);
+        m_table_colors.replace (i, setting_color);
       }
 
     update_colors ();
 
     // Icon size in the toolbar.
 
-    int icon_size_settings = settings->value ("toolbar_icon_size", 0).toInt ();
+    int size_idx = settings->value (global_icon_size).toInt ();
+    size_idx = (size_idx > 0) - (size_idx < 0) + 1;  // Make valid index from 0 to 2
+
     QStyle *st = style ();
-    int icon_size = st->pixelMetric (QStyle::PM_ToolBarIconSize);
-
-    // FIXME: Magic numbers.  Use enum?
-
-    if (icon_size_settings == 1)
-      icon_size = st->pixelMetric (QStyle::PM_LargeIconSize);
-    else if (icon_size_settings == -1)
-      icon_size = st->pixelMetric (QStyle::PM_SmallIconSize);
-
+    int icon_size = st->pixelMetric (global_icon_sizes[size_idx]);
     m_tool_bar->setIconSize (QSize (icon_size, icon_size));
   }
 
@@ -1345,17 +1418,49 @@ namespace octave
   }
 
   void
-  variable_editor::variable_destroyed (QObject *)
+  variable_editor::variable_destroyed (QObject *obj)
   {
+    // Invalidate the focus-restoring widget pointer if currently active.
+    if (m_focus_widget_vdw == obj)
+      {
+        m_focus_widget = nullptr;
+        m_focus_widget_vdw = nullptr;
+      }
+
+    // If no variable pages remain, deactivate the tool bar.
     QList<variable_dock_widget *> vdwlist = findChildren<variable_dock_widget *> ();
     if (vdwlist.isEmpty ())
       m_tool_bar->setEnabled (false);
+
+    QFocusEvent ev (QEvent::FocusIn);
+    focusInEvent (&ev);
   }
 
   void
   variable_editor::variable_focused (const QString &name)
   {
     m_current_focus_vname = name;
+
+    // focusWidget() appears lost in transition to/from main window
+    // so keep a record of the widget.
+
+    QWidget *current = QApplication::focusWidget ();
+    m_focus_widget = nullptr;
+    m_focus_widget_vdw = nullptr;
+    if (current != nullptr)
+      {
+        QList<variable_dock_widget *> vdwlist = findChildren<variable_dock_widget *> ();
+        for (int i = 0; i < vdwlist.size (); i++)
+          {
+            variable_dock_widget *vdw = vdwlist.at (i);
+            if (vdw->isAncestorOf (current))
+              {
+                m_focus_widget = current;
+                m_focus_widget_vdw = vdw;
+                break;
+              }
+          }
+      }
   }
 
   void
@@ -1409,14 +1514,6 @@ namespace octave
     emit updated ();
   }
 
-  void
-  variable_editor::pasteTableClipboard (void)
-  {
-    emit paste_table_clipboard_signal ();
-
-    emit updated ();
-  }
-
   void
   variable_editor::levelUp (void)
   {
@@ -1453,13 +1550,11 @@ namespace octave
 
     if (m_table_colors.length () > 4 && m_alternate_rows)
       {
-        m_stylesheet
-          += "QTableView::item:alternate{ background-color: "
-          + m_table_colors[4].name () +" }";
+        m_stylesheet += "QTableView::item:alternate{ background-color: "
+                        + m_table_colors[4].name () +" }";
 
-        m_stylesheet
-          += "QTableView::item:alternate:selected{ background-color: "
-          + m_table_colors[3].name () +" }";
+        m_stylesheet += "QTableView::item:alternate:selected{ background-color: "
+                        + m_table_colors[3].name () +" }";
       }
 
     QList<QTableView *> viewlist = findChildren<QTableView *> ();
@@ -1504,48 +1599,44 @@ namespace octave
 
     m_tool_bar->setWindowTitle (tr ("Variable Editor Toolbar"));
 
+    resource_manager& rmgr = m_octave_qobj.get_resource_manager ();
+
     QAction *action;
-    action = add_tool_bar_button (resource_manager::icon ("document-save"),
-                                  tr ("Save"), this, SLOT (save ()));
+    action = add_tool_bar_button (rmgr.icon ("document-save"), tr ("Save"),
+                                  this, SLOT (save ()));
+    addAction (action);
+    action->setShortcutContext (Qt::WidgetWithChildrenShortcut);
     action->setShortcuts (QKeySequence::Save);
     action->setStatusTip(tr("Save variable to a file"));
 
     m_tool_bar->addSeparator ();
 
-    action = add_tool_bar_button (resource_manager::icon ("edit-cut"),
-                                  tr ("Cut"), this, SLOT (cutClipboard ()));
+    action = add_tool_bar_button (rmgr.icon ("edit-cut"), tr ("Cut"),
+                                  this, SLOT (cutClipboard ()));
     action->setStatusTip(tr("Cut data to clipboard"));
 
-    action = add_tool_bar_button (resource_manager::icon ("edit-copy"),
-                                  tr ("Copy"), this, SLOT (copyClipboard ()));
+    action = add_tool_bar_button (rmgr.icon ("edit-copy"), tr ("Copy"),
+                                  this, SLOT (copyClipboard ()));
     action->setStatusTip(tr("Copy data to clipboard"));
 
-    action = add_tool_bar_button (resource_manager::icon ("edit-paste"),
-                                  tr ("Paste"), this, SLOT (pasteClipboard ()));
+    action = add_tool_bar_button (rmgr.icon ("edit-paste"), tr ("Paste"),
+                                  this, SLOT (pasteClipboard ()));
     action->setStatusTip(tr("Paste clipboard into variable data"));
 
-    // FIXME: Different icon for Paste Table?
-
-    action = add_tool_bar_button (resource_manager::icon ("edit-paste"),
-                                  tr ("Paste Table"),
-                                  this, SLOT (pasteTableClipboard ()));
-    action->setStatusTip(tr("Another paste clipboard into variable data"));
-
     m_tool_bar->addSeparator ();
 
     // FIXME: Add a print item?
     // QAction *print_action; /icons/fileprint.png
     // m_tool_bar->addSeparator ();
 
-    action = new QAction (resource_manager::icon ("plot-xy-curve"),
-                          tr ("Plot"), m_tool_bar);
+    action = new QAction (rmgr.icon ("plot-xy-curve"), tr ("Plot"), m_tool_bar);
     action->setToolTip (tr ("Plot Selected Data"));
     QToolButton *plot_tool_button = new HoverToolButton (m_tool_bar);
     plot_tool_button->setDefaultAction (action);
 
     plot_tool_button->setText (tr ("Plot"));
     plot_tool_button->setToolTip (tr ("Plot selected data"));
-    plot_tool_button->setIcon (resource_manager::icon ("plot-xy-curve"));
+    plot_tool_button->setIcon (rmgr.icon ("plot-xy-curve"));
 
     plot_tool_button->setPopupMode (QToolButton::InstantPopup);
 
@@ -1564,19 +1655,19 @@ namespace octave
 
     m_tool_bar->addSeparator ();
 
-    action = add_tool_bar_button (resource_manager::icon ("go-up"),
-                                  tr ("Up"), this, SLOT (levelUp ()));
+    action = add_tool_bar_button (rmgr.icon ("go-up"), tr ("Up"), this,
+                                  SLOT (levelUp ()));
     action->setStatusTip(tr("Go one level up in variable hierarchy"));
 
     // The QToolButton mouse-clicks change active window, so connect all
-    // HoverToolButton and RuternFocusToolButton objects to the mechanism
+    // HoverToolButton and ReturnFocusToolButton objects to the mechanism
     // that restores active window and focus before acting.
     QList<HoverToolButton *> hbuttonlist
       = m_tool_bar->findChildren<HoverToolButton *> (""
 #if defined (QOBJECT_FINDCHILDREN_ACCEPTS_FINDCHILDOPTIONS)
                                                      , Qt::FindDirectChildrenOnly
 #endif
-                                                     );
+                                                    );
     for (int i = 0; i < hbuttonlist.size (); i++)
       {
         connect (hbuttonlist.at (i), SIGNAL (hovered_signal ()),
@@ -1590,7 +1681,7 @@ namespace octave
 #if defined (QOBJECT_FINDCHILDREN_ACCEPTS_FINDCHILDOPTIONS)
                                                            , Qt::FindDirectChildrenOnly
 #endif
-                                                           );
+                                                          );
     for (int i = 0; i < rfbuttonlist.size (); i++)
       {
         connect (rfbuttonlist.at (i), SIGNAL (about_to_activate ()),
