@@ -1,26 +1,27 @@
-/*
-
-Copyright (C) 1996-2018 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
-
-// Author: James R. Van Zandt <jrv@vanzandt.mv.com>
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -29,8 +30,9 @@ along with Octave; see the file COPYING.  If not, see
 #include <cstring>
 
 #include <iomanip>
-#include <iostream>
+#include <istream>
 #include <limits>
+#include <ostream>
 #include <sstream>
 #include <string>
 #include <vector>
@@ -50,7 +52,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "str-vec.h"
 
 #include "Cell.h"
-#include "call-stack.h"
 #include "defaults.h"
 #include "defun.h"
 #include "error.h"
@@ -64,12 +65,12 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-map.h"
 #include "ov-cell.h"
 #include "ov-class.h"
-#include "ov-fcn-inline.h"
 #include "ov.h"
 #include "ovl.h"
 #include "pager.h"
 #include "parse.h"
-#include "pt-exp.h"
+#include "pt-eval.h"
+#include "stack-frame.h"
 #include "sysdep.h"
 #include "unwind-prot.h"
 #include "utils.h"
@@ -510,6 +511,9 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
   if (read_mat5_tag (is, swap, type, element_length, is_small_data_element))
     return retval;                      // EOF
 
+  octave::interpreter& interp
+    = octave::__get_interpreter__ ("read_mat5_binary_element");
+
   if (type == miCOMPRESSED)
     {
 #if defined (HAVE_ZLIB)
@@ -844,7 +848,7 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
         if (! is)
           goto data_read_error;
 
-        // Octave can handle both "/" and "\" as a directory seperator
+        // Octave can handle both "/" and "\" as a directory separator
         // and so can ignore the separator field of m0.  I think the
         // sentinel field is also save to ignore.
         octave_scalar_map m0 = tc2.scalar_map_value ();
@@ -857,12 +861,16 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
         if (ftype == "simple" || ftype == "scopedfunction")
           {
             if (fpath.empty ())
-              // We have a builtin function
-              tc = make_fcn_handle (fname);
+              {
+                octave::tree_evaluator& tw = interp.get_evaluator ();
+
+                // We have a builtin function
+                // XXX FCN_HANDLE: SIMPLE/SCOPED
+                tc = tw.make_fcn_handle (fname);
+              }
             else
               {
-                std::string mroot =
-                  m0.contents ("matlabroot").string_value ();
+                std::string mroot = m0.contents ("matlabroot").string_value ();
 
                 if ((fpath.length () >= mroot.length ())
                     && fpath.substr (0, mroot.length ()) == mroot
@@ -893,6 +901,7 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
                                                         "", "", fname);
 
                         if (ov_fcn.is_defined ())
+                          // XXX FCN_HANDLE: SIMPLE/SCOPED
                           tc = octave_value (new octave_fcn_handle (ov_fcn, fname));
                       }
                     else
@@ -903,12 +912,11 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
                         names.push_back (fname + ".mex");
                         names.push_back (fname + ".m");
 
-                        octave::load_path& lp = octave::__get_load_path__ ("read_mat5_binary_element");
+                        octave::load_path& lp = interp.get_load_path ();
 
                         octave::directory_path p (lp.system_path ());
 
-                        str =
-                          octave::sys::env::make_absolute (p.find_first_of (names));
+                        str = octave::sys::env::make_absolute (p.find_first_of (names));
 
                         size_t xpos
                           = str.find_last_of (octave::sys::file_ops::dir_sep_chars ());
@@ -920,11 +928,13 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
                                                         "", "", fname);
 
                         if (ov_fcn.is_defined ())
+                          // XXX FCN_HANDLE: SIMPLE/SCOPED
                           tc = octave_value (new octave_fcn_handle (ov_fcn, fname));
                         else
                           {
-                            warning ("load: can't find the file %s",
-                                     fpath.c_str ());
+                            warning_with_id ("Octave:load:file-not-found",
+                                             "load: can't find the file %s",
+                                             fpath.c_str ());
                             goto skip_ahead;
                           }
                       }
@@ -941,11 +951,13 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
                                                     "", "", fname);
 
                     if (ov_fcn.is_defined ())
+                      // XXX FCN_HANDLE: SIMPLE/SCOPED
                       tc = octave_value (new octave_fcn_handle (ov_fcn, fname));
                     else
                       {
-                        warning ("load: can't find the file %s",
-                                 fpath.c_str ());
+                        warning_with_id ("Octave:load:file-not-found",
+                                         "load: can't find the file %s",
+                                         fpath.c_str ());
                         goto skip_ahead;
                       }
                   }
@@ -953,7 +965,8 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
           }
         else if (ftype == "nested")
           {
-            warning ("load: can't load nested function");
+            warning_with_id ("Octave:load:unsupported-type",
+                             "load: can't load nested function");
             goto skip_ahead;
           }
         else if (ftype == "anonymous")
@@ -962,59 +975,58 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
               = m1.contents ("workspace").scalar_map_value ();
             uint32NDArray MCOS = m2.contents ("MCOS").uint32_array_value ();
             octave_idx_type off
-              = static_cast<octave_idx_type>(MCOS(4).double_value ());
+              = static_cast<octave_idx_type> (MCOS(4).double_value ());
             m2 = subsys_ov.scalar_map_value ();
             m2 = m2.contents ("MCOS").scalar_map_value ();
             tc2 = m2.contents ("MCOS").cell_value ()(1 + off).cell_value ()(1);
             m2 = tc2.scalar_map_value ();
 
-            octave::unwind_protect_safe frame;
-
-            // Set up temporary scope to use for evaluating the text
-            // that defines the anonymous function.
-
-            octave::symbol_table& symtab
-              = octave::__get_symbol_table__ ("read_mat5_binary_element");
-
-            octave::symbol_scope local_scope;
-
-            symtab.set_scope (local_scope);
-
-            octave::call_stack& cs
-              = octave::__get_call_stack__ ("read_mat5_binary_element");
-            cs.push (local_scope, 0);
-            frame.add_method (cs, &octave::call_stack::pop);
+            octave::stack_frame::local_vars_map local_vars;
 
             if (m2.nfields () > 0)
               {
                 octave_value tmp;
 
-                for (octave_map::iterator p0 = m2.begin () ;
-                     p0 != m2.end (); p0++)
+                for (auto p0 = m2.begin (); p0 != m2.end (); p0++)
                   {
                     std::string key = m2.key (p0);
                     octave_value val = m2.contents (p0);
 
-                    local_scope.assign (key, val, 0);
+                    local_vars[key] = val;
                   }
               }
 
+            // Set up temporary scope to use for evaluating the text
+            // that defines the anonymous function so that we don't
+            // pick up values of random variables that might be in the
+            // current scope.
+
+            octave::tree_evaluator& tw = interp.get_evaluator ();
+            tw.push_dummy_scope ("read_mat5_binary_element");
+
+            octave::unwind_action act ([&tw] () { tw.pop_scope (); });
+
+            // FIXME: If evaluation of the string gives us an anonymous
+            // function handle object, then why extract the function and
+            // create a new anonymous function object?  Why not just
+            // attach the workspace values to the object returned by
+            // eval_string?  This code is also is duplicated in
+            // anon_fcn_handle::parse_anon_fcn_handle.
+
             int parse_status;
-            octave_value anon_fcn_handle =
-              octave::eval_string (fname.substr (4), true, parse_status);
+            octave_value anon_fcn_handle
+              = interp.eval_string (fname.substr (4), true, parse_status);
 
             if (parse_status != 0)
               error ("load: failed to load anonymous function handle");
 
-            octave_fcn_handle *fh =
-              anon_fcn_handle.fcn_handle_value ();
+            octave_fcn_handle *fh = anon_fcn_handle.fcn_handle_value ();
 
             if (! fh)
               error ("load: failed to load anonymous function handle");
 
-            tc = new octave_fcn_handle (fh->fcn_val (), "@<anonymous>");
-
-            frame.run ();
+            // XXX FCN_HANDLE: ANONYMOUS
+            tc = octave_value (new octave_fcn_handle (fh->fcn_val (), local_vars));
           }
         else
           error ("load: invalid function handle type");
@@ -1181,59 +1193,50 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
 
         if (isclass)
           {
-            if (classname == "inline")
+            octave::cdef_manager& cdm = interp.get_cdef_manager ();
+
+            if (cdm.find_class (classname, false, true).ok ())
               {
-                // inline is not an object in Octave but rather an
-                // overload of a function handle.  Special case.
-                tc =
-                  new octave_fcn_inline (m.contents ("expr")(0).string_value (),
-                                         m.contents ("args")(0).string_value ());
+                tc = m;
+                warning_with_id ("Octave:load:classdef-to-struct",
+                                 "load: classdef element has been converted to a struct");
               }
             else
               {
-                cdef_manager& cdm
-                  = octave::__get_cdef_manager__ ("read_mat5_binary_element");
+                octave_class *cls
+                  = new octave_class (m, classname,
+                                      std::list<std::string> ());
 
-                if (cdm.find_class (classname, false, true).ok ())
+                if (cls->reconstruct_exemplar ())
                   {
-                    tc = m;
-                    warning ("load: classdef element has been converted to a struct");
+
+                    if (! cls->reconstruct_parents ())
+                      warning_with_id ("Octave:load:classdef-object-inheritance",
+                                       "load: unable to reconstruct object inheritance");
+
+                    tc = cls;
+
+                    octave::load_path& lp = interp.get_load_path ();
+
+                    if (lp.find_method (classname, "loadobj") != "")
+                      {
+                        try
+                          {
+                            octave_value_list tmp = octave::feval ("loadobj", tc, 1);
+
+                            tc = tmp(0);
+                          }
+                        catch (const octave::execution_exception&)
+                          {
+                            goto data_read_error;
+                          }
+                      }
                   }
                 else
                   {
-                    octave_class *cls
-                      = new octave_class (m, classname,
-                                          std::list<std::string> ());
-
-                    if (cls->reconstruct_exemplar ())
-                      {
-
-                        if (! cls->reconstruct_parents ())
-                          warning ("load: unable to reconstruct object inheritance");
-
-                        tc = cls;
-
-                        octave::load_path& lp = octave::__get_load_path__ ("read_mat5_binary_element");
-
-                        if (lp.find_method (classname, "loadobj") != "")
-                          {
-                            try
-                              {
-                                octave_value_list tmp = octave::feval ("loadobj", tc, 1);
-
-                                tc = tmp(0);
-                              }
-                            catch (const octave::execution_exception&)
-                              {
-                                goto data_read_error;
-                              }
-                          }
-                      }
-                    else
-                      {
-                        tc = m;
-                        warning ("load: element has been converted to a structure");
-                      }
+                    tc = m;
+                    warning_with_id ("Octave:load:classdef-to-struct",
+                                     "load: element has been converted to a structure");
                   }
               }
           }
@@ -1427,7 +1430,8 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
                       }
 
                     if (found_big_char)
-                      warning ("load: can not read non-ASCII portions of UTF characters; replacing unreadable characters with '?'");
+                      warning_with_id ("Octave:load:unsupported-utf-char",
+                               "load: can not read non-ASCII portions of UTF characters; replacing unreadable characters with '?'");
                   }
                 else if (type == miUTF8)
                   {
@@ -1444,7 +1448,8 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
 
                     if (utf8_multi_byte)
                       {
-                        warning ("load: can not read multi-byte encoded UTF8 characters; replacing unreadable characters with '?'");
+                        warning_with_id ("Octave:load:unsupported-utf-char",
+                                         "load: can not read multi-byte encoded UTF8 characters; replacing unreadable characters with '?'");
                         for (octave_idx_type i = 0; i < n; i++)
                           {
                             unsigned char a
@@ -1475,7 +1480,8 @@ data_read_error:
   error ("load: trouble reading binary file '%s'", filename.c_str ());
 
 skip_ahead:
-  warning ("load: skipping over '%s'", retval.c_str ());
+  warning_with_id ("Octave:load:skip-unsupported-element",
+                   "load: skipping over '%s'", retval.c_str ());
   is.seekg (pos + static_cast<std::streamoff> (element_length));
   return read_mat5_binary_element (is, filename, swap, global, tc);
 }
@@ -1496,9 +1502,9 @@ read_mat5_binary_file_header (std::istream& is, bool& swap, bool quiet,
   is.read (reinterpret_cast<char *> (&magic), 2);
 
   if (magic == 0x4d49)
-    swap = 0;
+    swap = false;
   else if (magic == 0x494d)
-    swap = 1;
+    swap = true;
   else
     {
       if (! quiet)
@@ -1511,8 +1517,9 @@ read_mat5_binary_file_header (std::istream& is, bool& swap, bool quiet,
     version = ((version >> 8) & 0xff) + ((version & 0xff) << 8);
 
   if (version != 1 && ! quiet)
-    warning ("load: found version %d binary MAT file, "
-             "but only prepared for version 1", version);
+    warning_with_id ("Octave:load:unsupported-version",
+                     "load: found version %d binary MAT file, but only prepared for version 1",
+                     version);
 
   if (swap)
     swap_bytes<8> (&subsys_offset, 1);
@@ -1606,8 +1613,8 @@ write_mat5_array (std::ostream& os, const NDArray& m, bool save_as_floats)
     {
       if (m.too_large_for_float ())
         {
-          warning ("save: some values too large to save as floats --");
-          warning ("save: saving as doubles instead");
+          warning_with_id ("Octave:save:too-large-for-float",
+                           "save: some values too large to save as floats -- saving as doubles instead");
         }
       else
         st = LS_FLOAT;
@@ -2011,7 +2018,7 @@ save_mat5_array_length (const float* /* val */, octave_idx_type nel, bool)
 
       // Round nel up to nearest even number of elements.
       // Take into account short tags for 4 byte elements.
-      return PAD ((nel > 0 && nel * size <= 4 ? 4 : 8) + nel * size);
+      return PAD ((nel * size <= 4 ? 4 : 8) + nel * size);
     }
   else
     return 8;
@@ -2180,8 +2187,7 @@ save_mat5_element_length (const octave_value& tc, const std::string& name,
       octave_idx_type nel = m.numel ();
 
       if (tc.is_inline_function ())
-        // length of "inline" is 6
-        ret += 8 + PAD (6 > max_namelen ? max_namelen : 6);
+        ret += 8 + PAD (6);  // length of "inline" is 6
       else if (tc.isobject ())
         {
           size_t classlen = tc.class_name ().length ();
@@ -2189,7 +2195,7 @@ save_mat5_element_length (const octave_value& tc, const std::string& name,
           ret += 8 + PAD (classlen > max_namelen ? max_namelen : classlen);
         }
 
-      for (octave_map::const_iterator i = m.begin (); i != m.end (); i++)
+      for (auto i = m.begin (); i != m.end (); i++)
         fieldcnt++;
 
       ret += 16 + fieldcnt * (max_namelen + 1);
@@ -2197,7 +2203,7 @@ save_mat5_element_length (const octave_value& tc, const std::string& name,
       for (octave_idx_type j = 0; j < nel; j++)
         {
 
-          for (octave_map::const_iterator i = m.begin (); i != m.end (); i++)
+          for (auto i = m.begin (); i != m.end (); i++)
             {
               const Cell elts = m.contents (i);
 
@@ -2230,8 +2236,9 @@ write_mat5_sparse_index_vector (std::ostream& os,
 static void
 warn_dim_too_large (const std::string& name)
 {
-  warning ("save: skipping %s: dimension too large for MAT format",
-           name.c_str ());
+  warning_with_id ("Octave:save:dimension-too-large",
+                   "save: skipping %s: dimension too large for MAT format",
+                   name.c_str ());
 }
 
 // save the data from TC along with the corresponding NAME on stream
@@ -2611,7 +2618,8 @@ save_mat5_binary_element (std::ostream& os,
 
       octave_map m;
 
-      octave::load_path& lp = octave::__get_load_path__ ("read_mat5_binary_element");
+      octave::load_path& lp
+        = octave::__get_load_path__ ("save_mat5_binary_element");
 
       if (tc.isobject ()
           && lp.find_method (tc.class_name (), "saveobj") != "")
