@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 2007-2017 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2007-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 /*
 
@@ -33,28 +36,32 @@ To initialize:
 #  include "config.h"
 #endif
 
+#include <string>
+
+#include "dMatrix.h"
+#include "file-stat.h"
+#include "oct-env.h"
+
 #include "build-env.h"
 #include "builtin-defun-decls.h"
 #include "defun-dld.h"
 #include "error.h"
-#include "file-stat.h"
 #include "graphics.h"
-#include "oct-env.h"
+#include "ov.h"
+#include "ovl.h"
 #include "parse.h"
 #include "utils.h"
 #include "variables.h"
 
 // PKG_ADD: if (__have_gnuplot__ ()) register_graphics_toolkit ("gnuplot"); endif
 
-static bool toolkit_loaded = false;
-
-class gnuplot_graphics_toolkit : public base_graphics_toolkit
+class gnuplot_graphics_toolkit : public octave::base_graphics_toolkit
 {
 public:
-  gnuplot_graphics_toolkit (void)
-    : base_graphics_toolkit ("gnuplot") { }
+  gnuplot_graphics_toolkit (octave::interpreter& interp)
+    : octave::base_graphics_toolkit ("gnuplot"), m_interpreter (interp) { }
 
-  ~gnuplot_graphics_toolkit (void) { }
+  ~gnuplot_graphics_toolkit (void) = default;
 
   bool is_valid (void) const { return true; }
 
@@ -67,8 +74,8 @@ public:
   {
     if (go.isa ("figure"))
       {
-        const figure::properties& props =
-          dynamic_cast<const figure::properties&> (go.get_properties ());
+        const figure::properties& props
+          = dynamic_cast<const figure::properties&> (go.get_properties ());
 
         send_quit (props.get___plot_stream__ ());
       }
@@ -80,8 +87,8 @@ public:
       {
         graphics_object obj (go);
 
-        figure::properties& props =
-          dynamic_cast<figure::properties&> (obj.get_properties ());
+        figure::properties& props
+          = dynamic_cast<figure::properties&> (obj.get_properties ());
 
         switch (id)
           {
@@ -99,9 +106,19 @@ public:
 
   void redraw_figure (const graphics_object& go) const
   {
-    octave_value_list args;
-    args(0) = go.get_handle ().as_octave_value ();
-    feval ("__gnuplot_drawnow__", args);
+    static bool drawnow_executing = false;
+
+    // Prevent recursion
+    if (! drawnow_executing)
+      {
+        octave::unwind_protect frame;
+        frame.protect_var (drawnow_executing);
+
+        drawnow_executing = true;
+        octave_value_list args;
+        args(0) = go.get_handle ().as_octave_value ();
+        octave::feval ("__gnuplot_drawnow__", args);
+      }
   }
 
   void print_figure (const graphics_object& go, const std::string& term,
@@ -114,7 +131,7 @@ public:
     args(2) = file;
     args(1) = term;
     args(0) = go.get_handle ().as_octave_value ();
-    feval ("__gnuplot_drawnow__", args);
+    octave::feval ("__gnuplot_drawnow__", args);
   }
 
   Matrix get_canvas_size (const graphics_handle&) const
@@ -131,39 +148,35 @@ public:
 
   void close (void)
   {
-    if (toolkit_loaded)
-      {
-        munlock ("__init_gnuplot__");
-
-        gtk_manager::unload_toolkit ("gnuplot");
-
-        toolkit_loaded = false;
-      }
+    if (m_interpreter.mislocked ("__init_gnuplot__"))
+      m_interpreter.munlock ("__init_gnuplot__");
   }
 
 private:
 
   void send_quit (const octave_value& pstream) const
   {
-    if (! pstream.is_empty ())
+    if (! pstream.isempty ())
       {
         octave_value_list args;
         Matrix fids = pstream.matrix_value ();
 
-        Ffputs (ovl (fids(0), "\nquit;\n"));
+        Ffputs (m_interpreter, ovl (fids(0), "\nquit;\n"));
 
-        Ffflush (ovl (fids(0)));
-        Fpclose (ovl (fids(0)));
+        Ffflush (m_interpreter, ovl (fids(0)));
+        Fpclose (m_interpreter, ovl (fids(0)));
 
         if (fids.numel () > 1)
           {
-            Fpclose (ovl (fids(1)));
+            Fpclose (m_interpreter, ovl (fids(1)));
 
             if (fids.numel () > 2)
               Fwaitpid (ovl (fids(2)));
           }
       }
   }
+
+  octave::interpreter& m_interpreter;
 };
 
 static bool
@@ -175,14 +188,15 @@ have_gnuplot_binary (void)
 
   try
     {
-      octave_value_list tmp = feval ("gnuplot_binary", octave_value_list ());
+      octave_value_list tmp
+        = octave::feval ("gnuplot_binary", octave_value_list ());
 
-      if (tmp(0).is_string ())
+      if (tmp(0).is_string () && ! tmp(0).isempty ())
         {
           std::string gnuplot_binary = tmp(0).string_value ();
 
           string_vector args (gnuplot_binary);
-          std::string gnuplot_path = search_path_for_file (path, args);
+          std::string gnuplot_path = octave::search_path_for_file (path, args);
 
           octave::sys::file_stat fs (gnuplot_path);
 
@@ -190,7 +204,7 @@ have_gnuplot_binary (void)
             {
               args[0] += exeext;
 
-              gnuplot_path = search_path_for_file (path, args);
+              gnuplot_path = octave::search_path_for_file (path, args);
 
               fs = octave::sys::file_stat (gnuplot_path);
             }
@@ -206,27 +220,25 @@ have_gnuplot_binary (void)
 
 // Initialize the gnuplot graphics toolkit.
 
-DEFUN_DLD (__init_gnuplot__, , ,
-           doc: /* -*- texinfo -*-
+DEFMETHOD_DLD (__init_gnuplot__, interp, , ,
+               doc: /* -*- texinfo -*-
 @deftypefn {} {} __init_gnuplot__ ()
 Undocumented internal function.
 @end deftypefn */)
 {
-  octave_value retval;
-
   if (! have_gnuplot_binary ())
     error ("__init_gnuplot__: the gnuplot program is not available, see 'gnuplot_binary'");
-  else if (! toolkit_loaded)
+  else if (! interp.mislocked ("__init_gnuplot__"))
     {
-      mlock ();
+      octave::gtk_manager& gtk_mgr = interp.get_gtk_manager ();
 
-      graphics_toolkit tk (new gnuplot_graphics_toolkit ());
-      gtk_manager::load_toolkit (tk);
+      octave::graphics_toolkit tk (new gnuplot_graphics_toolkit (interp));
+      gtk_mgr.load_toolkit (tk);
 
-      toolkit_loaded = true;
+      interp.mlock ();
     }
 
-  return retval;
+  return octave_value_list ();
 }
 
 DEFUN_DLD (__have_gnuplot__, , ,
@@ -235,11 +247,7 @@ DEFUN_DLD (__have_gnuplot__, , ,
 Undocumented internal function.
 @end deftypefn */)
 {
-  octave_value retval;
-
-  retval = have_gnuplot_binary ();
-
-  return retval;
+  return ovl (have_gnuplot_binary ());
 }
 
 /*
