@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.7.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +31,10 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
+   especially those whose name start with YY_ or yy_.  They are
+   private implementation details that can be changed or removed.  */
+
 #ifndef YY_OCTAVE_LIBINTERP_PARSE_TREE_OCT_PARSE_H_INCLUDED
 # define YY_OCTAVE_LIBINTERP_PARSE_TREE_OCT_PARSE_H_INCLUDED
 /* Debug traces.  */
@@ -48,92 +53,100 @@
 extern int octave_debug;
 #endif
 
-/* Token type.  */
+/* Token kinds.  */
 #ifndef OCTAVE_TOKENTYPE
 # define OCTAVE_TOKENTYPE
   enum octave_tokentype
   {
-    ADD_EQ = 258,
-    SUB_EQ = 259,
-    MUL_EQ = 260,
-    DIV_EQ = 261,
-    LEFTDIV_EQ = 262,
-    POW_EQ = 263,
-    EMUL_EQ = 264,
-    EDIV_EQ = 265,
-    ELEFTDIV_EQ = 266,
-    EPOW_EQ = 267,
-    AND_EQ = 268,
-    OR_EQ = 269,
-    EXPR_AND_AND = 270,
-    EXPR_OR_OR = 271,
-    EXPR_AND = 272,
-    EXPR_OR = 273,
-    EXPR_NOT = 274,
-    EXPR_LT = 275,
-    EXPR_LE = 276,
-    EXPR_EQ = 277,
-    EXPR_NE = 278,
-    EXPR_GE = 279,
-    EXPR_GT = 280,
-    LEFTDIV = 281,
-    EMUL = 282,
-    EDIV = 283,
-    ELEFTDIV = 284,
-    EPLUS = 285,
-    EMINUS = 286,
-    HERMITIAN = 287,
-    TRANSPOSE = 288,
-    PLUS_PLUS = 289,
-    MINUS_MINUS = 290,
-    POW = 291,
-    EPOW = 292,
-    NUM = 293,
-    IMAG_NUM = 294,
-    STRUCT_ELT = 295,
-    NAME = 296,
-    END = 297,
-    DQ_STRING = 298,
-    SQ_STRING = 299,
-    FOR = 300,
-    PARFOR = 301,
-    WHILE = 302,
-    DO = 303,
-    UNTIL = 304,
-    IF = 305,
-    ELSEIF = 306,
-    ELSE = 307,
-    SWITCH = 308,
-    CASE = 309,
-    OTHERWISE = 310,
-    BREAK = 311,
-    CONTINUE = 312,
-    FUNC_RET = 313,
-    UNWIND = 314,
-    CLEANUP = 315,
-    TRY = 316,
-    CATCH = 317,
-    GLOBAL = 318,
-    PERSISTENT = 319,
-    FCN_HANDLE = 320,
-    CLASSDEF = 321,
-    PROPERTIES = 322,
-    METHODS = 323,
-    EVENTS = 324,
-    ENUMERATION = 325,
-    METAQUERY = 326,
-    SUPERCLASSREF = 327,
-    FQ_IDENT = 328,
-    GET = 329,
-    SET = 330,
-    FCN = 331,
-    LEXICAL_ERROR = 332,
-    END_OF_INPUT = 333,
-    INPUT_FILE = 334,
-    UNARY = 335
+    OCTAVE_EMPTY = -2,
+    OCTAVE_EOF = 0,                /* "end of file"  */
+    OCTAVE_error = 256,            /* error  */
+    OCTAVE_UNDEF = 257,            /* "invalid token"  */
+    ADD_EQ = 258,                  /* ADD_EQ  */
+    SUB_EQ = 259,                  /* SUB_EQ  */
+    MUL_EQ = 260,                  /* MUL_EQ  */
+    DIV_EQ = 261,                  /* DIV_EQ  */
+    LEFTDIV_EQ = 262,              /* LEFTDIV_EQ  */
+    POW_EQ = 263,                  /* POW_EQ  */
+    EMUL_EQ = 264,                 /* EMUL_EQ  */
+    EDIV_EQ = 265,                 /* EDIV_EQ  */
+    ELEFTDIV_EQ = 266,             /* ELEFTDIV_EQ  */
+    EPOW_EQ = 267,                 /* EPOW_EQ  */
+    AND_EQ = 268,                  /* AND_EQ  */
+    OR_EQ = 269,                   /* OR_EQ  */
+    EXPR_AND_AND = 270,            /* EXPR_AND_AND  */
+    EXPR_OR_OR = 271,              /* EXPR_OR_OR  */
+    EXPR_AND = 272,                /* EXPR_AND  */
+    EXPR_OR = 273,                 /* EXPR_OR  */
+    EXPR_NOT = 274,                /* EXPR_NOT  */
+    EXPR_LT = 275,                 /* EXPR_LT  */
+    EXPR_LE = 276,                 /* EXPR_LE  */
+    EXPR_EQ = 277,                 /* EXPR_EQ  */
+    EXPR_NE = 278,                 /* EXPR_NE  */
+    EXPR_GE = 279,                 /* EXPR_GE  */
+    EXPR_GT = 280,                 /* EXPR_GT  */
+    LEFTDIV = 281,                 /* LEFTDIV  */
+    EMUL = 282,                    /* EMUL  */
+    EDIV = 283,                    /* EDIV  */
+    ELEFTDIV = 284,                /* ELEFTDIV  */
+    EPLUS = 285,                   /* EPLUS  */
+    EMINUS = 286,                  /* EMINUS  */
+    HERMITIAN = 287,               /* HERMITIAN  */
+    TRANSPOSE = 288,               /* TRANSPOSE  */
+    PLUS_PLUS = 289,               /* PLUS_PLUS  */
+    MINUS_MINUS = 290,             /* MINUS_MINUS  */
+    POW = 291,                     /* POW  */
+    EPOW = 292,                    /* EPOW  */
+    NUM = 293,                     /* NUM  */
+    IMAG_NUM = 294,                /* IMAG_NUM  */
+    STRUCT_ELT = 295,              /* STRUCT_ELT  */
+    NAME = 296,                    /* NAME  */
+    END = 297,                     /* END  */
+    DQ_STRING = 298,               /* DQ_STRING  */
+    SQ_STRING = 299,               /* SQ_STRING  */
+    FOR = 300,                     /* FOR  */
+    PARFOR = 301,                  /* PARFOR  */
+    WHILE = 302,                   /* WHILE  */
+    DO = 303,                      /* DO  */
+    UNTIL = 304,                   /* UNTIL  */
+    IF = 305,                      /* IF  */
+    ELSEIF = 306,                  /* ELSEIF  */
+    ELSE = 307,                    /* ELSE  */
+    SWITCH = 308,                  /* SWITCH  */
+    CASE = 309,                    /* CASE  */
+    OTHERWISE = 310,               /* OTHERWISE  */
+    BREAK = 311,                   /* BREAK  */
+    CONTINUE = 312,                /* CONTINUE  */
+    FUNC_RET = 313,                /* FUNC_RET  */
+    UNWIND = 314,                  /* UNWIND  */
+    CLEANUP = 315,                 /* CLEANUP  */
+    TRY = 316,                     /* TRY  */
+    CATCH = 317,                   /* CATCH  */
+    GLOBAL = 318,                  /* GLOBAL  */
+    PERSISTENT = 319,              /* PERSISTENT  */
+    FCN_HANDLE = 320,              /* FCN_HANDLE  */
+    CLASSDEF = 321,                /* CLASSDEF  */
+    PROPERTIES = 322,              /* PROPERTIES  */
+    METHODS = 323,                 /* METHODS  */
+    EVENTS = 324,                  /* EVENTS  */
+    ENUMERATION = 325,             /* ENUMERATION  */
+    METAQUERY = 326,               /* METAQUERY  */
+    SUPERCLASSREF = 327,           /* SUPERCLASSREF  */
+    FQ_IDENT = 328,                /* FQ_IDENT  */
+    GET = 329,                     /* GET  */
+    SET = 330,                     /* SET  */
+    FCN = 331,                     /* FCN  */
+    LEXICAL_ERROR = 332,           /* LEXICAL_ERROR  */
+    END_OF_INPUT = 333,            /* END_OF_INPUT  */
+    INPUT_FILE = 334,              /* INPUT_FILE  */
+    UNARY = 335                    /* UNARY  */
   };
+  typedef enum octave_tokentype octave_token_kind_t;
 #endif
-/* Tokens.  */
+/* Token kinds.  */
+#define OCTAVE_EOF 0
+#define OCTAVE_error 256
+#define OCTAVE_UNDEF 257
 #define ADD_EQ 258
 #define SUB_EQ 259
 #define MUL_EQ 260
@@ -215,10 +228,9 @@ extern int octave_debug;
 
 /* Value type.  */
 #if ! defined OCTAVE_STYPE && ! defined OCTAVE_STYPE_IS_DECLARED
-
 union OCTAVE_STYPE
 {
-#line 136 "libinterp/parse-tree/oct-parse.yy" /* yacc.c:1909  */
+#line 131 "/home/jwe/src/octave-stable/libinterp/parse-tree/oct-parse.yy"
 
   int dummy_type;
 
@@ -236,7 +248,8 @@ union OCTAVE_STYPE
   octave::tree_expression *tree_expression_type;
   octave::tree_constant *tree_constant_type;
   octave::tree_fcn_handle *tree_fcn_handle_type;
-  octave::tree_funcall *tree_funcall_type;
+  octave::tree_superclass_ref *tree_superclass_ref_type;
+  octave::tree_metaclass_query *tree_metaclass_query_type;
   octave::tree_function_def *tree_function_def_type;
   octave::tree_anon_fcn_handle *tree_anon_fcn_handle_type;
   octave::tree_identifier *tree_identifier_type;
@@ -276,9 +289,9 @@ union OCTAVE_STYPE
   octave::tree_classdef_enum_list* tree_classdef_enum_list_type;
   octave::tree_classdef_enum_block* tree_classdef_enum_block_type;
 
-#line 280 "libinterp/parse-tree/oct-parse.h" /* yacc.c:1909  */
-};
+#line 293 "libinterp/parse-tree/oct-parse.h"
 
+};
 typedef union OCTAVE_STYPE OCTAVE_STYPE;
 # define OCTAVE_STYPE_IS_TRIVIAL 1
 # define OCTAVE_STYPE_IS_DECLARED 1
@@ -293,10 +306,13 @@ enum { YYPUSH_MORE = 4 };
 
 typedef struct octave_pstate octave_pstate;
 
+
 int octave_parse (octave::base_parser& parser);
-int octave_push_parse (octave_pstate *ps, int pushed_char, OCTAVE_STYPE const *pushed_val, octave::base_parser& parser);
+int octave_push_parse (octave_pstate *ps,
+                  int pushed_char, OCTAVE_STYPE const *pushed_val, octave::base_parser& parser);
 int octave_pull_parse (octave_pstate *ps, octave::base_parser& parser);
-octave_pstate * octave_pstate_new (void);
+octave_pstate *octave_pstate_new (void);
 void octave_pstate_delete (octave_pstate *ps);
 
+
 #endif /* !YY_OCTAVE_LIBINTERP_PARSE_TREE_OCT_PARSE_H_INCLUDED  */
