@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 1996-2018 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -33,6 +36,10 @@ along with Octave; see the file COPYING.  If not, see
 #include "stat-wrappers.h"
 #include "strmode-wrapper.h"
 
+#if defined (OCTAVE_USE_WINDOWS_API)
+#  include "lo-regexp.h"
+#endif
+
 namespace octave
 {
   namespace sys
@@ -186,12 +193,21 @@ namespace octave
 
           std::string full_file_name = sys::file_ops::tilde_expand (file_name);
 
-#if defined (__WIN32__)
-          // Remove trailing slash.
-          if (full_file_name.length () > 1
-              && sys::file_ops::is_dir_sep (full_file_name.back ())
-              && ! (full_file_name.length () == 3 && full_file_name[1] == ':'))
+#if defined (OCTAVE_USE_WINDOWS_API)
+          full_file_name = sys::canonicalize_file_name (full_file_name);
+
+          // Remove trailing slashes
+          while (full_file_name.length () > 1
+              && sys::file_ops::is_dir_sep (full_file_name.back ()))
             full_file_name.pop_back ();
+
+          // If path is a root (like "C:" or "\\SERVER\share"), add a
+          // trailing backslash.
+          // FIXME: Does this pattern match all possible UNC roots?
+          octave::regexp pat (R"(^\\\\[\w-]*\\[\w-]*$)");
+          if ((full_file_name.length () == 2 && full_file_name[1] == ':')
+              || pat.is_match (full_file_name))
+            full_file_name += "\\";
 #endif
 
           const char *cname = full_file_name.c_str ();
